@@ -4,20 +4,21 @@
  *
  * Copyright (C) 2002-2005 Paolo Maggi
  * Copyright (C) 2009 Steve Frécinaux
+ * Copyright (C) 2010-2014 Garrett Regier
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU Library General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ * libpeas is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU Library General Public License for more details.
+ * libpeas is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- *  You should have received a copy of the GNU Library General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -36,7 +37,7 @@
 #include "peas-extension.h"
 #include "peas-dirs.h"
 #include "peas-debug.h"
-#include "peas-helpers.h"
+#include "peas-utils.h"
 
 /**
  * SECTION:peas-engine
@@ -57,11 +58,6 @@
  *   </listitem>
  * </itemizedlist>
  **/
-G_DEFINE_TYPE (PeasEngine, peas_engine, G_TYPE_OBJECT);
-
-static PeasEngine *default_engine = NULL;
-static gboolean shutdown = FALSE;
-static GHashTable *loaders = NULL;
 
 /* Signals */
 enum {
@@ -75,15 +71,26 @@ enum {
   PROP_0,
   PROP_PLUGIN_LIST,
   PROP_LOADED_PLUGINS,
+  PROP_NONGLOBAL_LOADERS,
   N_PROPERTIES
 };
 
 static guint signals[LAST_SIGNAL];
 static GParamSpec *properties[N_PROPERTIES] = { NULL };
 
-typedef struct _LoaderInfo {
+typedef struct _GlobalLoaderInfo {
   PeasPluginLoader *loader;
   PeasObjectModule *module;
+
+  guint enabled : 1;
+  guint failed : 1;
+} GlobalLoaderInfo;
+
+typedef struct _LoaderInfo {
+  PeasPluginLoader *loader;
+
+  guint enabled : 1;
+  guint failed : 1;
 } LoaderInfo;
 
 typedef struct _SearchPath {
@@ -92,13 +99,26 @@ typedef struct _SearchPath {
 } SearchPath;
 
 struct _PeasEnginePrivate {
-  GList *search_paths;
+  LoaderInfo loaders[PEAS_UTILS_N_LOADERS];
 
+  GList *search_paths;
   GList *plugin_list;
 
   guint in_dispose : 1;
+  guint use_nonglobal_loaders : 1;
 };
 
+G_DEFINE_TYPE_WITH_PRIVATE (PeasEngine, peas_engine, G_TYPE_OBJECT)
+
+#define GET_PRIV(o) \
+  (peas_engine_get_instance_private (o))
+
+static gboolean shutdown = FALSE;
+static PeasEngine *default_engine = NULL;
+
+static GMutex loaders_lock;
+static GlobalLoaderInfo loaders[PEAS_UTILS_N_LOADERS];
+
 static void peas_engine_load_plugin_real   (PeasEngine     *engine,
                                             PeasPluginInfo *info);
 static void peas_engine_unload_plugin_real (PeasEngine     *engine,
@@ -110,6 +130,7 @@ load_plugin_info (PeasEngine  *engine,
                   const gchar *module_dir,
                   const gchar *data_dir)
 {
+  PeasEnginePrivate *priv = GET_PRIV (engine);
   PeasPluginInfo *info;
   const gchar *module_name;
 
@@ -127,11 +148,12 @@ load_plugin_info (PeasEngine  *engine,
    * drop this one (user plugins override system plugins) */
   module_name = peas_plugin_info_get_module_name (info);
   if (peas_engine_get_plugin_info (engine, module_name) != NULL)
-    _peas_plugin_info_unref (info);
+    {
+      _peas_plugin_info_unref (info);
+    }
   else
     {
-      engine->priv->plugin_list = g_list_prepend (engine->priv->plugin_list,
-                                                  info);
+      priv->plugin_list = g_list_prepend (priv->plugin_list, info);
 
       g_object_notify_by_pspec (G_OBJECT (engine),
                                 properties[PROP_PLUGIN_LIST]);
@@ -192,11 +214,12 @@ load_dir_real (PeasEngine  *engine,
 void
 peas_engine_rescan_plugins (PeasEngine *engine)
 {
+  PeasEnginePrivate *priv = GET_PRIV (engine);
   GList *item;
 
   g_return_if_fail (PEAS_IS_ENGINE (engine));
 
-  if (engine->priv->search_paths == NULL)
+  if (priv->search_paths == NULL)
     {
       g_debug ("No search paths where provided");
       return;
@@ -205,7 +228,7 @@ peas_engine_rescan_plugins (PeasEngine *engine)
   g_object_freeze_notify (G_OBJECT (engine));
 
   /* Go and read everything from the provided search paths */
-  for (item = engine->priv->search_paths; item != NULL; item = item->next)
+  for (item = priv->search_paths; item != NULL; item = item->next)
     {
       SearchPath *sp = (SearchPath *) item->data;
       load_dir_real (engine, sp->module_dir, sp->data_dir, 1);
@@ -220,6 +243,7 @@ peas_engine_insert_search_path (PeasEngine  *engine,
                                 const gchar *module_dir,
                                 const gchar *data_dir)
 {
+  PeasEnginePrivate *priv = GET_PRIV (engine);
   SearchPath *sp;
 
   g_return_if_fail (PEAS_IS_ENGINE (engine));
@@ -229,9 +253,7 @@ peas_engine_insert_search_path (PeasEngine  *engine,
   sp->module_dir = g_strdup (module_dir);
   sp->data_dir = g_strdup (data_dir ? data_dir : module_dir);
 
-  engine->priv->search_paths = g_list_insert (engine->priv->search_paths,
-                                              sp,
-                                              position);
+  priv->search_paths = g_list_insert (priv->search_paths, sp, position);
 
   g_object_freeze_notify (G_OBJECT (engine));
   load_dir_real (engine, sp->module_dir, sp->data_dir, 1);
@@ -290,71 +312,38 @@ peas_engine_prepend_search_path (PeasEngine  *engine,
   peas_engine_insert_search_path (engine, 0, module_dir, data_dir);
 }
 
-static guint
-hash_lowercase (gconstpointer data)
-{
-  gchar *lowercase;
-  guint ret;
-
-  lowercase = g_ascii_strdown ((const gchar *) data, -1);
-  ret = g_str_hash (lowercase);
-  g_free (lowercase);
-
-  return ret;
-}
-
-static gboolean
-equal_lowercase (const gchar *a,
-                 const gchar *b)
-{
-  return g_ascii_strcasecmp (a, b) == 0;
-}
-
 static void
-loader_destroy (LoaderInfo *info)
+default_engine_weak_notify (gpointer    unused,
+                            PeasEngine *engine)
 {
-  if (!info)
-    return;
-
-  if (info->loader)
-    {
-      g_object_add_weak_pointer (G_OBJECT (info->loader),
-                                 (gpointer *) &info->loader);
-
-      g_object_unref (info->loader);
-      g_assert (info->loader == NULL);
-    }
-
-  g_free (info);
+  g_warn_if_fail (g_atomic_pointer_compare_and_exchange (&default_engine,
+                                                         engine, NULL));
 }
 
 static void
 peas_engine_init (PeasEngine *engine)
 {
+  PeasEnginePrivate *priv = GET_PRIV (engine);
+
   /* Set the default engine here and not in constructor() to make sure
    * that if a plugin is loaded and calls peas_engine_get_default()
    * that this engine is returned and not another.
+   *
+   * While peas_engine_get_default() is not thread-safe
+   * we are using atomics here to try and prevent bad
+   * things from happening.
    */
-  if (default_engine == NULL)
+  if (g_atomic_pointer_compare_and_exchange (&default_engine, NULL, engine))
     {
-      default_engine = engine;
-      g_object_add_weak_pointer (G_OBJECT (engine),
-                                 (gpointer *) &default_engine);
+      g_object_weak_ref (G_OBJECT (engine),
+                         (GWeakNotify) default_engine_weak_notify,
+                         NULL);
     }
 
-  engine->priv = G_TYPE_INSTANCE_GET_PRIVATE (engine,
-                                              PEAS_TYPE_ENGINE,
-                                              PeasEnginePrivate);
+  priv->in_dispose = FALSE;
 
-  engine->priv->in_dispose = FALSE;
-}
-
-static void
-loader_garbage_collect (const gchar *id,
-                        LoaderInfo  *info)
-{
-  if (info != NULL && info->loader != NULL)
-    peas_plugin_loader_garbage_collect (info->loader);
+  /* The C plugin loader is always enabled */
+  priv->loaders[peas_utils_get_loader_id ("C")].enabled = TRUE;
 }
 
 /**
@@ -369,11 +358,18 @@ loader_garbage_collect (const gchar *id,
 void
 peas_engine_garbage_collect (PeasEngine *engine)
 {
+  PeasEnginePrivate *priv = GET_PRIV (engine);
+  gint i;
+
   g_return_if_fail (PEAS_IS_ENGINE (engine));
 
-  g_hash_table_foreach (loaders,
-                        (GHFunc) loader_garbage_collect,
-                        NULL);
+  for (i = 0; i < G_N_ELEMENTS (loaders); ++i)
+    {
+      LoaderInfo *loader_info = &priv->loaders[i];
+
+      if (loader_info->loader != NULL)
+        peas_plugin_loader_garbage_collect (loader_info->loader);
+    }
 }
 
 static GObject *
@@ -388,6 +384,9 @@ peas_engine_constructor (GType                  type,
                "plugins engine as modules are not supported.");
     }
 
+  /* Don't need to use atomics as peas_engine_shutdown()
+   * is private API and as such is not multithread-safe
+   */
   if (shutdown)
     {
       g_error ("libpeas cannot create a plugin engine "
@@ -406,6 +405,7 @@ peas_engine_set_property (GObject      *object,
                           GParamSpec   *pspec)
 {
   PeasEngine *engine = PEAS_ENGINE (object);
+  PeasEnginePrivate *priv = GET_PRIV (engine);
 
   switch (prop_id)
     {
@@ -413,6 +413,9 @@ peas_engine_set_property (GObject      *object,
       peas_engine_set_loaded_plugins (engine,
                                       (const gchar **) g_value_get_boxed (value));
       break;
+    case PROP_NONGLOBAL_LOADERS:
+      priv->use_nonglobal_loaders = g_value_get_boolean (value);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -426,6 +429,7 @@ peas_engine_get_property (GObject    *object,
                           GParamSpec *pspec)
 {
   PeasEngine *engine = PEAS_ENGINE (object);
+  PeasEnginePrivate *priv = GET_PRIV (engine);
 
   switch (prop_id)
     {
@@ -437,6 +441,9 @@ peas_engine_get_property (GObject    *object,
       g_value_take_boxed (value,
                           (gconstpointer) peas_engine_get_loaded_plugins (engine));
       break;
+    case PROP_NONGLOBAL_LOADERS:
+      g_value_set_boolean (value, priv->use_nonglobal_loaders);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -447,12 +454,15 @@ static void
 peas_engine_dispose (GObject *object)
 {
   PeasEngine *engine = PEAS_ENGINE (object);
+  PeasEnginePrivate *priv = GET_PRIV (engine);
   GList *item;
+  gint i;
 
-  engine->priv->in_dispose = TRUE;
+  /* See peas_engine_unload_plugin_real() */
+  priv->in_dispose = TRUE;
 
   /* First unload all the plugins */
-  for (item = engine->priv->plugin_list; item; item = item->next)
+  for (item = priv->plugin_list; item; item = item->next)
     {
       PeasPluginInfo *info = PEAS_PLUGIN_INFO (item->data);
 
@@ -460,6 +470,14 @@ peas_engine_dispose (GObject *object)
         peas_engine_unload_plugin (engine, info);
     }
 
+  /* Then destroy the plugin loaders */
+  for (i = 0; i < G_N_ELEMENTS (priv->loaders); ++i)
+    {
+      LoaderInfo *loader_info = &priv->loaders[i];
+
+      g_clear_object (&loader_info->loader);
+    }
+
   G_OBJECT_CLASS (peas_engine_parent_class)->dispose (object);
 }
 
@@ -467,14 +485,15 @@ static void
 peas_engine_finalize (GObject *object)
 {
   PeasEngine *engine = PEAS_ENGINE (object);
+  PeasEnginePrivate *priv = GET_PRIV (engine);
   GList *item;
 
   /* free the infos */
-  g_list_free_full (engine->priv->plugin_list,
+  g_list_free_full (priv->plugin_list,
                     (GDestroyNotify) _peas_plugin_info_unref);
 
   /* free the search path list */
-  for (item = engine->priv->search_paths; item; item = item->next)
+  for (item = priv->search_paths; item; item = item->next)
     {
       SearchPath *sp = (SearchPath *) item->data;
 
@@ -483,7 +502,7 @@ peas_engine_finalize (GObject *object)
       g_slice_free (SearchPath, sp);
     }
 
-  g_list_free (engine->priv->search_paths);
+  g_list_free (priv->search_paths);
 
   G_OBJECT_CLASS (peas_engine_parent_class)->finalize (object);
 }
@@ -548,6 +567,24 @@ peas_engine_class_init (PeasEngineClass *klass)
                         G_PARAM_READWRITE |
                         G_PARAM_STATIC_STRINGS);
 
+  /**
+   * PeasEngine:nonglobal-loaders:
+   *
+   * If non-global plugin loaders should be used.
+   *
+   * See peas_engine_new_with_nonglobal_loaders() for more information.
+   *
+   * Since: 1.14
+   */
+  properties[PROP_NONGLOBAL_LOADERS] =
+    g_param_spec_boolean ("nonglobal-loaders",
+                          "Non-global Loaders",
+                          "Use non-global plugin loaders",
+                          FALSE,
+                          G_PARAM_READWRITE |
+                          G_PARAM_CONSTRUCT_ONLY |
+                          G_PARAM_STATIC_STRINGS);
+
   /**
    * PeasEngine::load-plugin:
    * @engine: A #PeasEngine.
@@ -598,145 +635,257 @@ peas_engine_class_init (PeasEngineClass *klass)
                   G_SIGNAL_TYPE_STATIC_SCOPE);
 
   g_object_class_install_properties (object_class, N_PROPERTIES, properties);
-  g_type_class_add_private (klass, sizeof (PeasEnginePrivate));
 
   /* We are doing some global initialization here as there is currently no
    * global init function for libpeas. */
   peas_debug_init ();
 
-  /* mapping from loadername -> loader object */
-  loaders = g_hash_table_new_full (hash_lowercase,
-                                   (GEqualFunc) equal_lowercase,
-                                   (GDestroyNotify) g_free,
-                                   (GDestroyNotify) loader_destroy);
-
   /* The C plugin loader is always enabled */
-  g_hash_table_insert (loaders, g_strdup ("C"), g_new0 (LoaderInfo, 1));
+  loaders[peas_utils_get_loader_id ("C")].enabled = TRUE;
 }
 
 static PeasObjectModule *
-load_module (const gchar *module_name,
-             const gchar *module_dir)
+get_plugin_loader_module (gint loader_id)
 {
-  PeasObjectModule *module;
+  GlobalLoaderInfo *global_loader_info = &loaders[loader_id];
+  gint i, j;
+  const gchar *loader_name;
+  gchar *module_name, *module_dir;
 
-  module = peas_object_module_new (module_name, module_dir, TRUE);
+  if (global_loader_info->module != NULL)
+    return global_loader_info->module;
 
-  if (!g_type_module_use (G_TYPE_MODULE (module)))
+  loader_name = peas_utils_get_loader_from_id (loader_id);
+  module_name = g_strconcat (loader_name, "loader", NULL);
+  module_dir = peas_dirs_get_plugin_loader_dir (loader_name);
+
+  /* Remove '.'s from the module name */
+  for (i = 0, j = 0; module_name[i] != '\0'; ++i)
     {
-      g_object_unref (module);
-      return NULL;
+      if (module_name[i] != '.')
+        module_name[j++] = module_name[i];
     }
 
-  return module;
+  module_name[j] = '\0';
+
+  /* Bind loaders globally, binding
+   * locally can break the plugin loaders
+   */
+  global_loader_info->module = peas_object_module_new_full (module_name,
+                                                            module_dir,
+                                                            TRUE, FALSE);
+
+  if (!g_type_module_use (G_TYPE_MODULE (global_loader_info->module)))
+    {
+      g_warning ("Could not load plugin loader '%s'", loader_name);
+      g_clear_object (&global_loader_info->module);
+    }
+
+  g_free (module_dir);
+  g_free (module_name);
+
+  return global_loader_info->module;
 }
 
 static PeasPluginLoader *
-get_plugin_loader (PeasEngine     *engine,
-                   PeasPluginInfo *info)
+create_plugin_loader (gint loader_id)
 {
-  LoaderInfo *loader_info;
-  gchar *loader_id, *module_name, *module_dir;
+  PeasPluginLoader *loader;
 
-  loader_info = (LoaderInfo *) g_hash_table_lookup (loaders, info->loader);
-
-  /* The loader has not been enabled. */
-  if (loader_info == NULL)
-    return NULL;
-
-  /* The loader has already been loaded. */
-  if (loader_info->loader != NULL)
-    return loader_info->loader;
-
-  /* Create the default C plugin loader. */
-  if (g_ascii_strcasecmp (info->loader, "C") == 0)
+  if (peas_utils_get_loader_id ("C") == loader_id)
     {
-      loader_info->loader = peas_plugin_loader_c_new ();
-      return loader_info->loader;
+      loader = peas_plugin_loader_c_new ();
+    }
+  else
+    {
+      PeasObjectModule *module;
+
+      module = get_plugin_loader_module (loader_id);
+      if (module == NULL)
+        return NULL;
+
+      loader = PEAS_PLUGIN_LOADER (
+            peas_object_module_create_object (module,
+                                              PEAS_TYPE_PLUGIN_LOADER,
+                                              0, NULL));
     }
 
-  loader_id = g_ascii_strdown (info->loader, -1);
-  module_name = g_strconcat (loader_id, "loader", NULL);
-  module_dir = peas_dirs_get_plugin_loaders_dir ();
-
-  loader_info->module = load_module (module_name, module_dir);
-
-  if (loader_info->module == NULL)
-    {
-      gchar *tmp = module_dir;
-
-      module_dir = g_build_filename (module_dir, loader_id, NULL);
-      loader_info->module = load_module (module_name, module_dir);
-
-      g_free (tmp);
-
-      if (loader_info->module == NULL)
-        {
-          g_warning ("Could not load plugin loader '%s'", info->loader);
-
-          g_free (module_dir);
-          g_free (module_name);
-          g_free (loader_id);
-          g_hash_table_insert (loaders, g_strdup (info->loader), NULL);
-          return NULL;
-        }
-    }
-
-  loader_info->loader = PEAS_PLUGIN_LOADER (
-        peas_object_module_create_object (loader_info->module,
-                                          PEAS_TYPE_PLUGIN_LOADER,
-                                          0, NULL));
-
-  g_type_module_unuse (G_TYPE_MODULE (loader_info->module));
-  g_free (module_dir);
-  g_free (module_name);
-  g_free (loader_id);
-
-  if (loader_info->loader == NULL ||
-      !peas_plugin_loader_initialize (loader_info->loader))
+  if (loader == NULL || !peas_plugin_loader_initialize (loader))
     {
       g_warning ("Loader '%s' is not a valid PeasPluginLoader instance",
-                 info->loader);
+                 peas_utils_get_loader_from_id (loader_id));
+      g_clear_object (&loader);
+    }
 
-      /* This will cause the loader to be unreffed if it exists */
-      g_hash_table_insert (loaders, g_strdup (info->loader), NULL);
+  return loader;
+}
+
+static PeasPluginLoader *
+get_local_plugin_loader (PeasEngine *engine,
+                         gint        loader_id)
+{
+  PeasEnginePrivate *priv = GET_PRIV (engine);
+  GlobalLoaderInfo *global_loader_info = &loaders[loader_id];
+  PeasPluginLoader *loader;
+
+  if (global_loader_info->failed)
+    return NULL;
+
+  if (global_loader_info->loader != NULL &&
+      (!priv->use_nonglobal_loaders ||
+       peas_plugin_loader_is_global (global_loader_info->loader)))
+    {
+      return g_object_ref (global_loader_info->loader);
+    }
+
+  loader = create_plugin_loader (loader_id);
+
+  if (loader == NULL)
+    {
+      global_loader_info->failed = TRUE;
       return NULL;
     }
 
+  if (!priv->use_nonglobal_loaders ||
+      peas_plugin_loader_is_global (loader))
+    {
+      global_loader_info->loader = g_object_ref (loader);
+    }
+
+  return loader;
+}
+
+static PeasPluginLoader *
+get_plugin_loader (PeasEngine *engine,
+                   gint        loader_id)
+{
+  PeasEnginePrivate *priv = GET_PRIV (engine);
+  LoaderInfo *loader_info = &priv->loaders[loader_id];
+  GlobalLoaderInfo *global_loader_info = &loaders[loader_id];
+
+  if (loader_info->loader != NULL || loader_info->failed)
+    return loader_info->loader;
+
+  g_mutex_lock (&loaders_lock);
+
+  if (!loader_info->enabled)
+    {
+      if (!global_loader_info->enabled)
+        {
+          g_warning ("The '%s' plugin loader has not been enabled",
+                     peas_utils_get_loader_from_id (loader_id));
+
+          g_mutex_unlock (&loaders_lock);
+          return NULL;
+        }
+
+      g_warning ("The '%s' plugin loader was not enabled "
+                 "for this engine. This will no longer be "
+                 "supported at some point in the future!",
+                 peas_utils_get_loader_from_id (loader_id));
+
+      g_mutex_unlock (&loaders_lock);
+
+      /* Avoid bypassing logic in peas_engine_enable_loader() */
+      peas_engine_enable_loader (engine,
+                                 peas_utils_get_loader_from_id (loader_id));
+      return get_plugin_loader (engine, loader_id);
+    }
+
+  loader_info->loader = get_local_plugin_loader (engine, loader_id);
+
+  if (loader_info->loader == NULL)
+    loader_info->failed = TRUE;
+
+  g_mutex_unlock (&loaders_lock);
   return loader_info->loader;
 }
 
 /**
  * peas_engine_enable_loader:
  * @engine: A #PeasEngine.
- * @loader_id: The id of the loader to enable.
+ * @loader_name: The name of the loader to enable.
  *
  * Enable a loader, enables a loader for plugins.
- * The C plugin loader is always enabled.
+ * The C plugin loader is always enabled. The other plugin
+ * loaders are: lua5.1 and python and python3.
  *
- * For instance, the following code will enable python plugin
- * from being loaded:
+ * For instance, the following code will enable python plugins
+ * to be loaded:
  * |[
  * peas_engine_enable_loader (engine, "python");
  * ]|
  *
- * Note: plugin loaders are shared across #PeasEngines so enabling
- *       a loader on one #PeasEngine will enable it on all #PeasEngines.
+ * Note: plugin loaders used to be shared across #PeasEngines so enabling
+ *       a loader on one #PeasEngine would enable it on all #PeasEngines.
+ *       This behavior has been kept to avoid breaking applications,
+ *       however a warning has been added to help applications transition.
  **/
 void
 peas_engine_enable_loader (PeasEngine  *engine,
-                           const gchar *loader_id)
+                           const gchar *loader_name)
 {
-  g_return_if_fail (PEAS_IS_ENGINE (engine));
-  g_return_if_fail (loader_id != NULL && *loader_id != '\0');
+  PeasEnginePrivate *priv = GET_PRIV (engine);
+  LoaderInfo *loader_info;
+  gint loader_id;
 
-  if (g_hash_table_lookup_extended (loaders, loader_id, NULL, NULL))
+  g_return_if_fail (PEAS_IS_ENGINE (engine));
+  g_return_if_fail (loader_name != NULL && *loader_name != '\0');
+
+  loader_id = peas_utils_get_loader_id (loader_name);
+
+  if (loader_id == -1)
+    {
+      g_warning ("Failed to enable unknown plugin loader '%s'", loader_name);
+      return;
+    }
+
+  loader_info = &priv->loaders[loader_id];
+  if (loader_info->enabled || loader_info->failed)
     return;
 
+  g_mutex_lock (&loaders_lock);
+
+  if (loaders[loader_id].enabled)
+    {
+      loader_info->enabled = TRUE;
+      g_mutex_unlock (&loaders_lock);
+      return;
+    }
+
+  /* Some tests check for mixed versions this way */
+  if (g_getenv ("PEAS_ALLOW_CONFLICTING_LOADERS") == NULL)
+    {
+      gint i;
+      const gint *loader_ids;
+
+      loader_ids = peas_utils_get_conflicting_loaders_from_id (loader_id);
+
+      /* Some loaders conflict with each other
+       * and cannot be used in the same process
+       */
+      for (i = 0; loader_ids[i] != -1; ++i)
+        {
+          if (!loaders[loader_ids[i]].enabled)
+            continue;
+
+          g_warning ("Cannot enable plugin loader '%s' as the "
+                     "'%s' plugin loader is already enabled.", loader_name,
+                     peas_utils_get_loader_from_id (loader_ids[i]));
+
+          loader_info->failed = TRUE;
+          g_mutex_unlock (&loaders_lock);
+          return;
+        }
+    }
+
   /* We do not load the plugin loader immediately and instead
    * load it in get_plugin_loader() so that it is loaded lazily.
    */
-  g_hash_table_insert (loaders, g_strdup (loader_id), g_new0 (LoaderInfo, 1));
+  loader_info->enabled = TRUE;
+  loaders[loader_id].enabled = TRUE;
+
+  g_mutex_unlock (&loaders_lock);
 }
 
 /**
@@ -752,9 +901,11 @@ peas_engine_enable_loader (PeasEngine  *engine,
 const GList *
 peas_engine_get_plugin_list (PeasEngine *engine)
 {
+  PeasEnginePrivate *priv = GET_PRIV (engine);
+
   g_return_val_if_fail (PEAS_IS_ENGINE (engine), NULL);
 
-  return engine->priv->plugin_list;
+  return priv->plugin_list;
 }
 
 static gint
@@ -779,12 +930,13 @@ PeasPluginInfo *
 peas_engine_get_plugin_info (PeasEngine  *engine,
                              const gchar *plugin_name)
 {
+  PeasEnginePrivate *priv = GET_PRIV (engine);
   GList *l;
 
   g_return_val_if_fail (PEAS_IS_ENGINE (engine), NULL);
   g_return_val_if_fail (plugin_name != NULL, NULL);
 
-  l = g_list_find_custom (engine->priv->plugin_list,
+  l = g_list_find_custom (priv->plugin_list,
                           plugin_name,
                           (GCompareFunc) compare_plugin_info_and_name);
 
@@ -837,17 +989,16 @@ peas_engine_load_plugin_real (PeasEngine     *engine,
         }
     }
 
-  loader = get_plugin_loader (engine, info);
+  loader = get_plugin_loader (engine, info->loader_id);
 
   if (loader == NULL)
     {
-      g_warning ("Could not find loader '%s' for plugin '%s'",
-                 info->loader, peas_plugin_info_get_module_name (info));
+      /* Already warned */
       g_set_error (&info->error,
                    PEAS_PLUGIN_INFO_ERROR,
                    PEAS_PLUGIN_INFO_ERROR_LOADER_NOT_FOUND,
                    _("Plugin loader '%s' was not found"),
-                   info->loader);
+                   peas_utils_get_loader_from_id (info->loader_id));
       goto error;
     }
 
@@ -908,6 +1059,7 @@ static void
 peas_engine_unload_plugin_real (PeasEngine     *engine,
                                 PeasPluginInfo *info)
 {
+  PeasEnginePrivate *priv = GET_PRIV (engine);
   GList *item;
   const gchar *module_name;
   PeasPluginLoader *loader;
@@ -921,7 +1073,7 @@ peas_engine_unload_plugin_real (PeasEngine     *engine,
 
   /* First unload all the dependant plugins */
   module_name = peas_plugin_info_get_module_name (info);
-  for (item = engine->priv->plugin_list; item; item = item->next)
+  for (item = priv->plugin_list; item; item = item->next)
     {
       PeasPluginInfo *other_info = PEAS_PLUGIN_INFO (item->data);
 
@@ -933,7 +1085,7 @@ peas_engine_unload_plugin_real (PeasEngine     *engine,
     }
 
   /* find the loader and tell it to gc and unload the plugin */
-  loader = get_plugin_loader (engine, info);
+  loader = get_plugin_loader (engine, info->loader_id);
 
   peas_plugin_loader_garbage_collect (loader);
   peas_plugin_loader_unload (loader, info);
@@ -943,7 +1095,7 @@ peas_engine_unload_plugin_real (PeasEngine     *engine,
   /* Don't notify while in dispose so the
    * loaded plugins can easily be kept in GSettings
    */
-  if (!engine->priv->in_dispose)
+  if (!priv->in_dispose)
     g_object_notify_by_pspec (G_OBJECT (engine),
                               properties[PROP_LOADED_PLUGINS]);
 }
@@ -999,7 +1151,7 @@ peas_engine_provides_extension (PeasEngine     *engine,
   if (!peas_plugin_info_is_loaded (info))
     return FALSE;
 
-  loader = get_plugin_loader (engine, info);
+  loader = get_plugin_loader (engine, info->loader_id);
   return peas_plugin_loader_provides_extension (loader, info, extension_type);
 }
 
@@ -1036,7 +1188,7 @@ peas_engine_create_extensionv (PeasEngine     *engine,
   g_return_val_if_fail (peas_plugin_info_is_loaded (info), NULL);
   g_return_val_if_fail (G_TYPE_IS_INTERFACE (extension_type), FALSE);
 
-  loader = get_plugin_loader (engine, info);
+  loader = get_plugin_loader (engine, info->loader_id);
   extension = peas_plugin_loader_create_extension (loader, info, extension_type,
                                                    n_parameters, parameters);
 
@@ -1085,8 +1237,9 @@ peas_engine_create_extension_valist (PeasEngine     *engine,
   g_return_val_if_fail (peas_plugin_info_is_loaded (info), NULL);
   g_return_val_if_fail (G_TYPE_IS_INTERFACE (extension_type), FALSE);
 
-  if (!_valist_to_parameter_list (extension_type, first_property,
-                                  var_args, &parameters, &n_parameters))
+  if (!peas_utils_valist_to_parameter_list (extension_type, first_property,
+                                            var_args, &parameters,
+                                            &n_parameters))
     {
       /* Already warned */
       return NULL;
@@ -1166,6 +1319,7 @@ peas_engine_create_extension (PeasEngine     *engine,
 gchar **
 peas_engine_get_loaded_plugins (PeasEngine *engine)
 {
+  PeasEnginePrivate *priv = GET_PRIV (engine);
   GArray *array;
   GList *pl;
 
@@ -1173,7 +1327,7 @@ peas_engine_get_loaded_plugins (PeasEngine *engine)
 
   array = g_array_new (TRUE, FALSE, sizeof (gchar *));
 
-  for (pl = engine->priv->plugin_list; pl; pl = pl->next)
+  for (pl = priv->plugin_list; pl; pl = pl->next)
     {
       PeasPluginInfo *info = (PeasPluginInfo *) pl->data;
       gchar *module_name;
@@ -1219,11 +1373,12 @@ void
 peas_engine_set_loaded_plugins (PeasEngine   *engine,
                                 const gchar **plugin_names)
 {
+  PeasEnginePrivate *priv = GET_PRIV (engine);
   GList *pl;
 
   g_return_if_fail (PEAS_IS_ENGINE (engine));
 
-  for (pl = engine->priv->plugin_list; pl; pl = pl->next)
+  for (pl = priv->plugin_list; pl; pl = pl->next)
     {
       PeasPluginInfo *info = (PeasPluginInfo *) pl->data;
       const gchar *module_name;
@@ -1261,6 +1416,29 @@ peas_engine_new (void)
   return PEAS_ENGINE (g_object_new (PEAS_TYPE_ENGINE, NULL));
 }
 
+/**
+ * peas_engine_new_with_nonglobal_loaders:
+ *
+ * Return a new instance of #PeasEngine which will use non-global
+ * plugin loaders instead of the default global ones. This allows
+ * multiple threads to each have a #PeasEngine and be used without
+ * internal locking.
+ *
+ * Note: due to CPython's GIL the python and python3
+ *       plugin loaders are always global.
+ *
+ * Returns: a new instance of #PeasEngine that uses non-global loaders.
+ *
+ * Since: 1.14
+ */
+PeasEngine *
+peas_engine_new_with_nonglobal_loaders (void)
+{
+  return PEAS_ENGINE (g_object_new (PEAS_TYPE_ENGINE,
+                                    "nonglobal-loaders", TRUE,
+                                    NULL));
+}
+
 /**
  * peas_engine_get_default:
  *
@@ -1268,6 +1446,9 @@ peas_engine_new (void)
  * If no #PeasEngine subclass has been instantiated yet, the first call
  * of this function will return a new instance of #PeasEngine.
  *
+ * Note: this function should never be used when multiple threads are
+ *       using libpeas API as it is not thread-safe.
+ *
  * Returns: (transfer none): the existing instance of #PeasEngine.
  */
 PeasEngine *
@@ -1275,28 +1456,58 @@ peas_engine_get_default (void)
 {
   /* peas_engine_new() will cause the default to be set for us */
   if (default_engine == NULL)
-    return peas_engine_new ();
+    {
+      PeasEngine *engine = peas_engine_new ();
+
+      if (engine != default_engine)
+        {
+          g_warning ("peas_engine_get_default() must not be called when "
+                     "multiple threads are using libpeas API");
+          g_object_unref (engine);
+        }
+    }
 
   return default_engine;
 }
 
-/*
+/* < private >
  * peas_engine_shutdown:
  *
  * Frees memory shared by PeasEngines.
  * No libpeas API should be called after calling this.
+ *
+ * Note: this is private API and as such is not thread-safe.
  */
 void
 peas_engine_shutdown (void)
 {
+  gint i;
+
   if (shutdown)
     return;
 
   shutdown = TRUE;
 
-  if (loaders != NULL)
+  g_mutex_lock (&loaders_lock);
+
+  for (i = 0; i < G_N_ELEMENTS (loaders); ++i)
     {
-      g_hash_table_destroy (loaders);
-      loaders = NULL;
+      GlobalLoaderInfo *loader_info = &loaders[i];
+
+      if (loader_info->loader != NULL)
+        {
+          g_object_add_weak_pointer (G_OBJECT (loader_info->loader),
+                                     (gpointer *) &loader_info->loader);
+
+          g_object_unref (loader_info->loader);
+          g_assert (loader_info->loader == NULL);
+        }
+
+      /* Don't bother unloading the
+       * module as it is always resident
+       */
+      loader_info->failed = TRUE;
     }
+
+  g_mutex_unlock (&loaders_lock);
 }
