@@ -5,19 +5,19 @@
  * Copyright (C) 2002-2008 Paolo Maggi
  * Copyright (C) 2009 Steve Frécinaux
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU Library General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ * libpeas is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU Library General Public License for more details.
+ * libpeas is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- *  You should have received a copy of the GNU Library General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -37,13 +37,11 @@
  * access the related #PeasPluginInfo, and especially the location where all
  * the data of your plugin lives.
  *
- * Non-C extensions will usually not inherit from this class: Python and
- * Seed plugins automatically get a "plugin_info" attribute that serves
+ * Non-C extensions will usually not inherit from this class: Python
+ * plugins automatically get a "plugin_info" attribute that serves
  * the same purpose.
  **/
 
-G_DEFINE_ABSTRACT_TYPE (PeasExtensionBase, peas_extension_base, G_TYPE_OBJECT)
-
 struct _PeasExtensionBasePrivate {
   PeasPluginInfo *info;
 };
@@ -58,6 +56,13 @@ enum {
 
 static GParamSpec *properties[N_PROPERTIES] = { NULL };
 
+G_DEFINE_ABSTRACT_TYPE_WITH_PRIVATE (PeasExtensionBase,
+                                     peas_extension_base,
+                                     G_TYPE_OBJECT)
+
+#define GET_PRIV(o) \
+  (peas_extension_base_get_instance_private (o))
+
 static void
 peas_extension_base_get_property (GObject    *object,
                                   guint       prop_id,
@@ -87,11 +92,12 @@ peas_extension_base_set_property (GObject      *object,
                                   GParamSpec   *pspec)
 {
   PeasExtensionBase *extbase = PEAS_EXTENSION_BASE (object);
+  PeasExtensionBasePrivate *priv = GET_PRIV (extbase);
 
   switch (prop_id)
     {
     case PROP_PLUGIN_INFO:
-      extbase->priv->info = g_value_get_boxed (value);
+      priv->info = g_value_get_boxed (value);
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -102,9 +108,6 @@ peas_extension_base_set_property (GObject      *object,
 static void
 peas_extension_base_init (PeasExtensionBase *extbase)
 {
-  extbase->priv = G_TYPE_INSTANCE_GET_PRIVATE (extbase,
-                                               PEAS_TYPE_EXTENSION_BASE,
-                                               PeasExtensionBasePrivate);
 }
 
 static void
@@ -134,7 +137,6 @@ peas_extension_base_class_init (PeasExtensionBaseClass *klass)
                          G_PARAM_STATIC_STRINGS);
 
   g_object_class_install_properties (object_class, N_PROPERTIES, properties);
-  g_type_class_add_private (klass, sizeof (PeasExtensionBasePrivate));
 }
 
 /**
@@ -149,9 +151,11 @@ peas_extension_base_class_init (PeasExtensionBaseClass *klass)
 PeasPluginInfo *
 peas_extension_base_get_plugin_info (PeasExtensionBase *extbase)
 {
+  PeasExtensionBasePrivate *priv = GET_PRIV (extbase);
+
   g_return_val_if_fail (PEAS_IS_EXTENSION_BASE (extbase), NULL);
 
-  return extbase->priv->info;
+  return priv->info;
 }
 
 /**
@@ -167,7 +171,9 @@ peas_extension_base_get_plugin_info (PeasExtensionBase *extbase)
 gchar *
 peas_extension_base_get_data_dir (PeasExtensionBase *extbase)
 {
+  PeasExtensionBasePrivate *priv = GET_PRIV (extbase);
+
   g_return_val_if_fail (PEAS_IS_EXTENSION_BASE (extbase), NULL);
 
-  return g_strdup (peas_plugin_info_get_data_dir (extbase->priv->info));
+  return g_strdup (peas_plugin_info_get_data_dir (priv->info));
 }
