@@ -8,19 +8,19 @@
  * Copyright (C) 2008 Jesse van den Kieboom
  * Copyright (C) 2010 Steve Frécinaux
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU Library General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ * libpeas is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU Library General Public License for more details.
+ * libpeas is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- *  You should have received a copy of the GNU Library General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -42,15 +42,14 @@
  * function.
  **/
 
-G_DEFINE_TYPE (PeasObjectModule, peas_object_module, G_TYPE_TYPE_MODULE)
-
-typedef void     (*PeasObjectModuleRegisterFunc) (PeasObjectModule *);
+typedef void (*PeasObjectModuleRegisterFunc) (PeasObjectModule *module);
 
 enum {
   PROP_0,
   PROP_MODULE_NAME,
   PROP_PATH,
   PROP_RESIDENT,
+  PROP_LOCAL_LINKAGE,
   N_PROPERTIES
 };
 
@@ -73,64 +72,79 @@ struct _PeasObjectModulePrivate {
   gchar *module_name;
 
   guint resident : 1;
+  guint local_linkage : 1;
 };
 
+G_DEFINE_TYPE_WITH_PRIVATE (PeasObjectModule,
+                            peas_object_module,
+                            G_TYPE_TYPE_MODULE)
+
+#define GET_PRIV(o) \
+  (peas_object_module_get_instance_private (o))
+
 static gboolean
 peas_object_module_load (GTypeModule *gmodule)
 {
   PeasObjectModule *module = PEAS_OBJECT_MODULE (gmodule);
+  PeasObjectModulePrivate *priv = GET_PRIV (module);
+  GModuleFlags flags = 0;
   gchar *path;
 
-  path = g_module_build_path (module->priv->path, module->priv->module_name);
+  if (priv->local_linkage)
+    flags = G_MODULE_BIND_LOCAL;
+
+  path = g_module_build_path (priv->path, priv->module_name);
   g_return_val_if_fail (path != NULL, FALSE);
 
-  /* g_module_build_path() will add G_MODULE_SUFFIX to the path, but otoh
-   * g_module_open() will only try to load the libtool archive if there is no
-   * suffix specified. So we remove G_MODULE_SUFFIX here (this allows
-   * uninstalled builds to load plugins as well, as there is only the .la file
-   * in the build directory) */
+  /* g_module_build_path() will add G_MODULE_SUFFIX to the path,
+   * however g_module_open() will only try to load the libtool archive
+   * if there is no suffix specified. So we remove G_MODULE_SUFFIX here
+   * which allows uninstalled builds to load plugins as well, as there
+   * is only the .la file in the build directory.
+   */
   if (G_MODULE_SUFFIX[0] != '\0' && g_str_has_suffix (path, "." G_MODULE_SUFFIX))
     path[strlen (path) - strlen (G_MODULE_SUFFIX) - 1] = '\0';
 
-  /* Bind symbols globally and immediately,
-   * binding locally broke the Python plugin loader.
-   */
-  module->priv->library = g_module_open (path, 0);
+  /* Bind symbols immediately to avoid errors long after loading */
+  priv->library = g_module_open (path, flags);
   g_free (path);
 
-  if (module->priv->library == NULL)
+  if (priv->library == NULL)
     {
-      g_warning ("%s: %s", module->priv->module_name, g_module_error ());
+      g_warning ("Failed to load module '%s': %s",
+                 priv->module_name, g_module_error ());
 
       return FALSE;
     }
 
-  /* extract symbols from the lib */
-  if (!g_module_symbol (module->priv->library,
+  /* Extract the required symbol from the library */
+  if (!g_module_symbol (priv->library,
                         "peas_register_types",
-                        (void *) &module->priv->register_func))
+                        (gpointer) &priv->register_func))
     {
-      g_warning ("%s: %s", module->priv->module_name, g_module_error ());
-      g_module_close (module->priv->library);
+      g_warning ("Failed to get 'peas_register_types' for module '%s': %s",
+                 priv->module_name, g_module_error ());
+      g_module_close (priv->library);
 
       return FALSE;
     }
 
-  /* symbol can still be NULL even though g_module_symbol
-   * returned TRUE */
-  if (module->priv->register_func == NULL)
+  /* The symbol can still be NULL even
+   * though g_module_symbol() returned TRUE
+   */
+  if (priv->register_func == NULL)
     {
-      g_warning ("%s: Symbol 'peas_register_types' is not defined",
-                 module->priv->module_name);
-      g_module_close (module->priv->library);
+      g_warning ("Invalid 'peas_register_types' in module '%s'",
+                 priv->module_name);
+      g_module_close (priv->library);
 
       return FALSE;
     }
 
-  if (module->priv->resident)
-    g_module_make_resident (module->priv->library);
+  if (priv->resident)
+    g_module_make_resident (priv->library);
 
-  module->priv->register_func (module);
+  priv->register_func (module);
 
   return TRUE;
 }
@@ -139,39 +153,44 @@ static void
 peas_object_module_unload (GTypeModule *gmodule)
 {
   PeasObjectModule *module = PEAS_OBJECT_MODULE (gmodule);
+  PeasObjectModulePrivate *priv = GET_PRIV (module);
+  InterfaceImplementation *impls;
+  guint i;
 
-  g_module_close (module->priv->library);
+  g_module_close (priv->library);
 
-  module->priv->library = NULL;
-  module->priv->register_func = NULL;
+  priv->library = NULL;
+  priv->register_func = NULL;
+
+  impls = (InterfaceImplementation *) priv->implementations->data;
+  for (i = 0; i < priv->implementations->len; ++i)
+    {
+      if (impls[i].destroy_func != NULL)
+        impls[i].destroy_func (impls[i].user_data);
+    }
+
+  g_array_remove_range (priv->implementations, 0,
+                        priv->implementations->len);
 }
 
 static void
 peas_object_module_init (PeasObjectModule *module)
 {
-  module->priv = G_TYPE_INSTANCE_GET_PRIVATE (module,
-                                              PEAS_TYPE_OBJECT_MODULE,
-                                              PeasObjectModulePrivate);
+  PeasObjectModulePrivate *priv = GET_PRIV (module);
 
-  module->priv->implementations = g_array_new (FALSE, FALSE, sizeof (InterfaceImplementation));
+  priv->implementations = g_array_new (FALSE, FALSE,
+                                       sizeof (InterfaceImplementation));
 }
 
 static void
 peas_object_module_finalize (GObject *object)
 {
   PeasObjectModule *module = PEAS_OBJECT_MODULE (object);
-  InterfaceImplementation *impls;
-  unsigned i;
+  PeasObjectModulePrivate *priv = GET_PRIV (module);
 
-  g_free (module->priv->path);
-  g_free (module->priv->module_name);
-
-  impls = (InterfaceImplementation *) module->priv->implementations->data;
-  for (i = 0; i < module->priv->implementations->len; ++i)
-    if (impls[i].destroy_func != NULL)
-      impls[i].destroy_func (impls[i].user_data);
-
-  g_array_unref (module->priv->implementations);
+  g_free (priv->path);
+  g_free (priv->module_name);
+  g_array_unref (priv->implementations);
 
   G_OBJECT_CLASS (peas_object_module_parent_class)->finalize (object);
 }
@@ -183,17 +202,21 @@ peas_object_module_get_property (GObject    *object,
                                  GParamSpec *pspec)
 {
   PeasObjectModule *module = PEAS_OBJECT_MODULE (object);
+  PeasObjectModulePrivate *priv = GET_PRIV (module);
 
   switch (prop_id)
     {
     case PROP_MODULE_NAME:
-      g_value_set_string (value, module->priv->module_name);
+      g_value_set_string (value, priv->module_name);
       break;
     case PROP_PATH:
-      g_value_set_string (value, module->priv->path);
+      g_value_set_string (value, priv->path);
       break;
     case PROP_RESIDENT:
-      g_value_set_boolean (value, module->priv->resident);
+      g_value_set_boolean (value, priv->resident);
+      break;
+    case PROP_LOCAL_LINKAGE:
+      g_value_set_boolean (value, priv->local_linkage);
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -208,19 +231,23 @@ peas_object_module_set_property (GObject      *object,
                                  GParamSpec   *pspec)
 {
   PeasObjectModule *module = PEAS_OBJECT_MODULE (object);
+  PeasObjectModulePrivate *priv = GET_PRIV (module);
 
   switch (prop_id)
     {
     case PROP_MODULE_NAME:
-      module->priv->module_name = g_value_dup_string (value);
+      priv->module_name = g_value_dup_string (value);
       g_type_module_set_name (G_TYPE_MODULE (object),
-                              module->priv->module_name);
+                              priv->module_name);
       break;
     case PROP_PATH:
-      module->priv->path = g_value_dup_string (value);
+      priv->path = g_value_dup_string (value);
       break;
     case PROP_RESIDENT:
-      module->priv->resident = g_value_get_boolean (value);
+      priv->resident = g_value_get_boolean (value);
+      break;
+    case PROP_LOCAL_LINKAGE:
+      priv->local_linkage = g_value_get_boolean (value);
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -268,8 +295,24 @@ peas_object_module_class_init (PeasObjectModuleClass *klass)
                           G_PARAM_CONSTRUCT_ONLY |
                           G_PARAM_STATIC_STRINGS);
 
+  /**
+   * PeasObjectModule:local-linkage
+   *
+   * This property indicates whether the module is loaded with
+   * local linkage, i.e. #G_MODULE_BIND_LOCAL.
+   *
+   * Since 1.14
+   */
+  properties[PROP_LOCAL_LINKAGE] =
+    g_param_spec_boolean ("local-linkage",
+                          "Local linkage",
+                          "Whether the module loaded with local linkage",
+                          FALSE,
+                          G_PARAM_READWRITE |
+                          G_PARAM_CONSTRUCT_ONLY |
+                          G_PARAM_STATIC_STRINGS);
+
   g_object_class_install_properties (object_class, N_PROPERTIES, properties);
-  g_type_class_add_private (klass, sizeof (PeasObjectModulePrivate));
 }
 
 /**
@@ -294,6 +337,33 @@ peas_object_module_new (const gchar *module_name,
                                            NULL));
 }
 
+/**
+ * peas_object_module_new_full: (skip)
+ * @module_name: The module name.
+ * @path: The path.
+ * @resident: If the module should be resident.
+ * @local_linkage: Whether to load the module with local linkage.
+ *
+ * Creates a new #PeasObjectModule with local linkage.
+ *
+ * Return value: a new #PeasObjectModule.
+ *
+ * Since 1.14
+ */
+PeasObjectModule *
+peas_object_module_new_full (const gchar *module_name,
+                             const gchar *path,
+                             gboolean     resident,
+                             gboolean     local_linkage)
+{
+  return PEAS_OBJECT_MODULE (g_object_new (PEAS_TYPE_OBJECT_MODULE,
+                                           "module-name", module_name,
+                                           "path", path,
+                                           "resident", resident,
+                                           "local-linkage", local_linkage,
+                                           NULL));
+}
+
 /**
  * peas_object_module_create_object: (skip)
  * @module: A #PeasObjectModule.
@@ -314,13 +384,14 @@ peas_object_module_create_object (PeasObjectModule *module,
                                   guint             n_parameters,
                                   GParameter       *parameters)
 {
+  PeasObjectModulePrivate *priv = GET_PRIV (module);
   guint i;
   InterfaceImplementation *impls;
 
   g_return_val_if_fail (PEAS_IS_OBJECT_MODULE (module), NULL);
 
-  impls = (InterfaceImplementation *) module->priv->implementations->data;
-  for (i = 0; i < module->priv->implementations->len; ++i)
+  impls = (InterfaceImplementation *) priv->implementations->data;
+  for (i = 0; i < priv->implementations->len; ++i)
     if (impls[i].iface_type == interface)
       return impls[i].func (n_parameters, parameters, impls[i].user_data);
 
@@ -340,13 +411,14 @@ gboolean
 peas_object_module_provides_object (PeasObjectModule *module,
                                     GType             interface)
 {
+  PeasObjectModulePrivate *priv = GET_PRIV (module);
   guint i;
   InterfaceImplementation *impls;
 
   g_return_val_if_fail (PEAS_IS_OBJECT_MODULE (module), FALSE);
 
-  impls = (InterfaceImplementation *) module->priv->implementations->data;
-  for (i = 0; i < module->priv->implementations->len; ++i)
+  impls = (InterfaceImplementation *) priv->implementations->data;
+  for (i = 0; i < priv->implementations->len; ++i)
     if (impls[i].iface_type == interface)
       return TRUE;
 
@@ -364,9 +436,11 @@ peas_object_module_provides_object (PeasObjectModule *module,
 const gchar *
 peas_object_module_get_path (PeasObjectModule *module)
 {
+  PeasObjectModulePrivate *priv = GET_PRIV (module);
+
   g_return_val_if_fail (PEAS_IS_OBJECT_MODULE (module), NULL);
 
-  return module->priv->path;
+  return priv->path;
 }
 
 /**
@@ -380,9 +454,11 @@ peas_object_module_get_path (PeasObjectModule *module)
 const gchar *
 peas_object_module_get_module_name (PeasObjectModule *module)
 {
+  PeasObjectModulePrivate *priv = GET_PRIV (module);
+
   g_return_val_if_fail (PEAS_IS_OBJECT_MODULE (module), NULL);
 
-  return module->priv->module_name;
+  return priv->module_name;
 }
 
 /**
@@ -396,9 +472,11 @@ peas_object_module_get_module_name (PeasObjectModule *module)
 GModule *
 peas_object_module_get_library (PeasObjectModule *module)
 {
+  PeasObjectModulePrivate *priv = GET_PRIV (module);
+
   g_return_val_if_fail (PEAS_IS_OBJECT_MODULE (module), NULL);
 
-  return module->priv->library;
+  return priv->library;
 }
 
 /**
@@ -426,15 +504,17 @@ peas_object_module_register_extension_factory (PeasObjectModule *module,
                                                gpointer          user_data,
                                                GDestroyNotify    destroy_func)
 {
+  PeasObjectModulePrivate *priv = GET_PRIV (module);
   InterfaceImplementation impl = { iface_type, factory_func, user_data, destroy_func };
 
   g_return_if_fail (PEAS_IS_OBJECT_MODULE (module));
+  g_return_if_fail (!peas_object_module_provides_object (module, iface_type));
   g_return_if_fail (factory_func != NULL);
 
   if (iface_type != PEAS_TYPE_PLUGIN_LOADER)
     g_return_if_fail (G_TYPE_IS_INTERFACE (iface_type));
 
-  g_array_append_val (module->priv->implementations, impl);
+  g_array_append_val (priv->implementations, impl);
 
   g_debug ("Registered extension for type '%s'", g_type_name (iface_type));
 }
@@ -457,7 +537,7 @@ create_gobject_from_type (guint       n_parameters,
    * Let's get rid of it if the actual type doesn't have such a
    * property to avoid a warning. */
   if (n_parameters > 0 &&
-      strcmp (parameters[n_parameters-1].name, "plugin-info") == 0 &&
+      strcmp (parameters[n_parameters - 1].name, "plugin-info") == 0 &&
       g_object_class_find_property (cls, "plugin-info") == NULL)
     n_parameters--;
 
@@ -483,6 +563,7 @@ peas_object_module_register_extension_type (PeasObjectModule *module,
                                             GType             extension_type)
 {
   g_return_if_fail (PEAS_IS_OBJECT_MODULE (module));
+  g_return_if_fail (!peas_object_module_provides_object (module, iface_type));
 
   if (iface_type != PEAS_TYPE_PLUGIN_LOADER)
     {
