@@ -4,19 +4,19 @@
  *
  * Copyright (C) 2008 - Jesse van den Kieboom
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU Library General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ * libpeas is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU Library General Public License for more details.
+ * libpeas is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- *  You should have received a copy of the GNU Library General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -29,46 +29,69 @@
 
 #include "peas-extension-base.h"
 #include "peas-object-module.h"
+#include "peas-plugin-info-priv.h"
+
+typedef struct {
+  GMutex lock;
 
-struct _PeasPluginLoaderCPrivate {
   GHashTable *loaded_plugins;
-};
+} PeasPluginLoaderCPrivate;
 
-G_DEFINE_TYPE (PeasPluginLoaderC, peas_plugin_loader_c, PEAS_TYPE_PLUGIN_LOADER)
+G_DEFINE_TYPE_WITH_PRIVATE (PeasPluginLoaderC,
+                            peas_plugin_loader_c,
+                            PEAS_TYPE_PLUGIN_LOADER)
+
+#define GET_PRIV(o) \
+  (peas_plugin_loader_c_get_instance_private (o))
+
+static
+G_DEFINE_QUARK (peas-extension-type, extension_type)
 
 static gboolean
 peas_plugin_loader_c_load (PeasPluginLoader *loader,
                            PeasPluginInfo   *info)
 {
   PeasPluginLoaderC *cloader = PEAS_PLUGIN_LOADER_C (loader);
-  PeasObjectModule *module;
-  const gchar *module_name;
+  PeasPluginLoaderCPrivate *priv = GET_PRIV (cloader);
 
-  module_name = peas_plugin_info_get_module_name (info);
-  module = (PeasObjectModule *) g_hash_table_lookup (cloader->priv->loaded_plugins,
-                                                     module_name);
+  g_mutex_lock (&priv->lock);
 
-  if (module == NULL)
+  if (!g_hash_table_lookup_extended (priv->loaded_plugins,
+                                     info->filename,
+                                     NULL, (gpointer *) &info->loader_data))
     {
+      const gchar *module_name, *module_dir;
+
+      module_name = peas_plugin_info_get_module_name (info);
+      module_dir = peas_plugin_info_get_module_dir (info);
+
       /* Force all C modules to be resident in case they
-       * use libraries that do not deal well with reloading
+       * use libraries that do not deal well with reloading.
+       * Furthermore, we use local linkage to improve module isolation.
        */
-      module = peas_object_module_new (module_name,
-                                       peas_plugin_info_get_module_dir (info),
-                                       TRUE);
+      info->loader_data = peas_object_module_new_full (module_name,
+                                                       module_dir,
+                                                       TRUE, TRUE);
 
-      g_hash_table_insert (cloader->priv->loaded_plugins,
-                           g_strdup (module_name), module);
+      if (!g_type_module_use (G_TYPE_MODULE (info->loader_data)))
+        g_clear_object (&info->loader_data);
+
+      g_hash_table_insert (priv->loaded_plugins,
+                           g_strdup (info->filename), info->loader_data);
     }
 
-  if (!g_type_module_use (G_TYPE_MODULE (module)))
-    {
-      g_object_unref (module);
-      g_hash_table_remove (cloader->priv->loaded_plugins, module_name);
-      return FALSE;
-    }
+  g_mutex_unlock (&priv->lock);
+  return info->loader_data != NULL;
+}
 
-  return TRUE;
+static void
+peas_plugin_loader_c_unload (PeasPluginLoader *loader,
+                             PeasPluginInfo   *info)
+{
+  /* Don't bother unloading the plugin's
+   * GTypeModule as it is always resident
+   */
+  info->loader_data = NULL;
 }
 
 static gboolean
@@ -76,15 +99,7 @@ peas_plugin_loader_c_provides_extension  (PeasPluginLoader *loader,
                                           PeasPluginInfo   *info,
                                           GType             exten_type)
 {
-  PeasPluginLoaderC *cloader = PEAS_PLUGIN_LOADER_C (loader);
-  PeasObjectModule *module;
-  const gchar *module_name;
-
-  module_name = peas_plugin_info_get_module_name (info);
-  module = (PeasObjectModule *) g_hash_table_lookup (cloader->priv->loaded_plugins,
-                                                     module_name);
-
-  return peas_object_module_provides_object (module, exten_type);
+  return peas_object_module_provides_object (info->loader_data, exten_type);
 }
 
 static PeasExtension *
@@ -94,22 +109,15 @@ peas_plugin_loader_c_create_extension (PeasPluginLoader *loader,
                                        guint             n_parameters,
                                        GParameter       *parameters)
 {
-  PeasPluginLoaderC *cloader = PEAS_PLUGIN_LOADER_C (loader);
-  PeasObjectModule *module;
   GParameter *exten_parameters;
   gpointer instance;
-  const gchar *module_name;
-
-  module_name = peas_plugin_info_get_module_name (info);
-  module = (PeasObjectModule *) g_hash_table_lookup (cloader->priv->loaded_plugins,
-                                                     module_name);
 
   /* We want to add a "plugin-info" property so we can pass it to
    * the extension if it inherits from PeasExtensionBase. No need to
    * actually "duplicate" the GValues, a memcpy is sufficient as the
    * source GValues are longer lived than our local copy.
    */
-  exten_parameters = g_new (GParameter, n_parameters + 1);
+  exten_parameters = g_newa (GParameter, n_parameters + 1);
   memcpy (exten_parameters, parameters, sizeof (GParameter) * n_parameters);
 
   /* Initialize our additional property.
@@ -121,13 +129,12 @@ peas_plugin_loader_c_create_extension (PeasPluginLoader *loader,
   g_value_init (&exten_parameters[n_parameters].value, PEAS_TYPE_PLUGIN_INFO);
   g_value_set_boxed (&exten_parameters[n_parameters].value, info);
 
-  instance = peas_object_module_create_object (module,
+  instance = peas_object_module_create_object (info->loader_data,
                                                exten_type,
                                                n_parameters + 1,
                                                exten_parameters);
 
   g_value_unset (&exten_parameters[n_parameters].value);
-  g_free (exten_parameters);
 
   if (instance == NULL)
     return NULL;
@@ -135,50 +142,36 @@ peas_plugin_loader_c_create_extension (PeasPluginLoader *loader,
   g_return_val_if_fail (G_IS_OBJECT (instance), NULL);
   g_return_val_if_fail (G_TYPE_CHECK_INSTANCE_TYPE (instance, exten_type), NULL);
 
-  /* As we do not instantiate a PeasExtensionWrapper, we have to remember
-   * somehow which interface we are instantiating, to make it possible to use
-   * the deprecated peas_extension_get_extension_type() method.
+  /* We have to remember which interface we are instantiating
+   * for the deprecated peas_extension_get_extension_type().
    */
-  g_object_set_data (instance, "peas-extension-type", GUINT_TO_POINTER (exten_type));
+  g_object_set_qdata (instance, extension_type_quark (),
+                      GSIZE_TO_POINTER (exten_type));
 
   return instance;
 }
 
 static void
-peas_plugin_loader_c_unload (PeasPluginLoader *loader,
-                             PeasPluginInfo   *info)
+peas_plugin_loader_c_init (PeasPluginLoaderC *cloader)
 {
-  PeasPluginLoaderC *cloader = PEAS_PLUGIN_LOADER_C (loader);
-  PeasObjectModule *module;
-  const gchar *module_name;
+  PeasPluginLoaderCPrivate *priv = GET_PRIV (cloader);
 
-  module_name = peas_plugin_info_get_module_name (info);
-  module = (PeasObjectModule *) g_hash_table_lookup (cloader->priv->loaded_plugins,
-                                                     module_name);
+  g_mutex_init (&priv->lock);
 
-  g_type_module_unuse (G_TYPE_MODULE (module));
-}
-
-static void
-peas_plugin_loader_c_init (PeasPluginLoaderC *self)
-{
-  self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self,
-                                            PEAS_TYPE_PLUGIN_LOADER_C,
-                                            PeasPluginLoaderCPrivate);
-
-  /* loaded_plugins maps PeasPluginInfo:module-name to a PeasObjectModule */
-  self->priv->loaded_plugins = g_hash_table_new_full (g_str_hash,
-                                                      g_str_equal,
-                                                      g_free,
-                                                      NULL);
+  /* loaded_plugins maps PeasPluginInfo:filename to a PeasObjectModule */
+  priv->loaded_plugins = g_hash_table_new_full (g_str_hash, g_str_equal,
+                                                g_free, NULL);
 }
 
 static void
 peas_plugin_loader_c_finalize (GObject *object)
 {
   PeasPluginLoaderC *cloader = PEAS_PLUGIN_LOADER_C (object);
+  PeasPluginLoaderCPrivate *priv = GET_PRIV (cloader);
 
-  g_hash_table_destroy (cloader->priv->loaded_plugins);
+  g_mutex_clear (&priv->lock);
+
+  g_hash_table_destroy (priv->loaded_plugins);
 
   G_OBJECT_CLASS (peas_plugin_loader_c_parent_class)->finalize (object);
 }
@@ -195,8 +188,6 @@ peas_plugin_loader_c_class_init (PeasPluginLoaderCClass *klass)
   loader_class->unload = peas_plugin_loader_c_unload;
   loader_class->provides_extension = peas_plugin_loader_c_provides_extension;
   loader_class->create_extension = peas_plugin_loader_c_create_extension;
-
-  g_type_class_add_private (object_class, sizeof (PeasPluginLoaderCPrivate));
 }
 
 /**
