@@ -4,19 +4,19 @@
  *
  * Copyright (C) 2010 - Garrett Regier
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU Library General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ * libpeas is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU Library General Public License for more details.
+ * libpeas is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- *  You should have received a copy of the GNU Library General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -41,14 +41,19 @@ testing_init (gint    *argc,
   if (initialized)
     return;
 
+  /* Must set environment variables first */
+  testing_util_envars ();
+
   gtk_test_init (argc, argv, NULL);
 
-  /* Must be after gtk_test_init() changes the log settings */
+  /* Must be called after gtk_test_init()
+   * because it changed the log settings
+   */
   testing_util_init ();
 
-  g_irepository_prepend_search_path (BUILDDIR "/libpeas-gtk");
-
-  g_irepository_require (g_irepository_get_default (), "PeasGtk", "1.0", 0, &error);
+  g_irepository_require_private (g_irepository_get_default (),
+                                 BUILDDIR "/libpeas-gtk",
+                                 "PeasGtk", "1.0", 0, &error);
   g_assert_no_error (error);
 
   initialized = TRUE;
