@@ -4,19 +4,19 @@
  *
  * Copyright (C) 2010 - Garrett Regier
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU Library General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ * libpeas is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU Library General Public License for more details.
+ * libpeas is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- *  You should have received a copy of the GNU Library General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -248,8 +248,7 @@ test_engine_not_loadable_plugin (PeasEngine *engine)
   GError *error = NULL;
   PeasPluginInfo *info;
 
-  testing_util_push_log_hook ("*libnot-loadable.so: cannot open shared "
-                              "object file: No such file or directory");
+  testing_util_push_log_hook ("Failed to load module 'not-loadable'*");
   testing_util_push_log_hook ("Error loading plugin 'not-loadable'");
 
   info = peas_engine_get_plugin_info (engine, "not-loadable");
@@ -381,43 +380,12 @@ test_engine_loaded_plugins (PeasEngine *engine)
 }
 
 static void
-test_engine_nonexistent_loader (PeasEngine *engine)
+test_engine_enable_unkown_loader (PeasEngine *engine)
 {
-  GError *error = NULL;
-  PeasPluginInfo *info;
+  testing_util_push_log_hook ("Failed to enable unknown "
+                              "plugin loader 'does-not-exist'");
 
-  testing_util_push_log_hook ("Could not load plugin loader 'does-not-exist'*");
-
-  info = peas_engine_get_plugin_info (engine, "nonexistent-loader");
   peas_engine_enable_loader (engine, "does-not-exist");
-
-  g_assert (!peas_engine_load_plugin (engine, info));
-  g_assert (!peas_plugin_info_is_loaded (info));
-  g_assert (!peas_plugin_info_is_available (info, &error));
-  g_assert_error (error, PEAS_PLUGIN_INFO_ERROR,
-                  PEAS_PLUGIN_INFO_ERROR_LOADER_NOT_FOUND);
-
-  g_error_free (error);
-}
-
-static void
-test_engine_disabled_loader (PeasEngine *engine)
-{
-  PeasPluginInfo *info;
-  GError *error = NULL;
-
-  testing_util_push_log_hook ("The 'disabled' plugin "
-                              "loader has not been enabled*");
-
-  info = peas_engine_get_plugin_info (engine, "disabled-loader");
-
-  g_assert (!peas_engine_load_plugin (engine, info));
-  g_assert (!peas_plugin_info_is_loaded (info));
-  g_assert (!peas_plugin_info_is_available (info, &error));
-  g_assert_error (error, PEAS_PLUGIN_INFO_ERROR,
-                  PEAS_PLUGIN_INFO_ERROR_LOADER_NOT_FOUND);
-
-  g_error_free (error);
 }
 
 static void
@@ -433,7 +401,6 @@ test_engine_nonexistent_search_path (PeasEngine *engine)
   peas_engine_add_search_path (engine, "/nowhere", NULL);
 }
 
-#if GLIB_CHECK_VERSION (2, 38, 0)
 static void
 test_engine_shutdown (void)
 {
@@ -456,7 +423,6 @@ test_engine_shutdown_subprocess (PeasEngine *engine)
     engine = peas_engine_new ();
     g_assert (engine == NULL);
 }
-#endif
 
 int
 main (int    argc,
@@ -490,16 +456,13 @@ main (int    argc,
 
   TEST ("loaded-plugins", loaded_plugins);
 
-  TEST ("nonexistent-loader", nonexistent_loader);
-  TEST ("disabled-loader", disabled_loader);
+  TEST ("enable-unkown-loader", enable_unkown_loader);
   TEST ("enable-loader-multiple-times", enable_loader_multiple_times);
 
   TEST ("nonexistent-search-path", nonexistent_search_path);
 
-#if GLIB_CHECK_VERSION (2, 38, 0)
   TEST_FUNC ("shutdown", shutdown);
   TEST ("shutdown/subprocess", shutdown_subprocess);
-#endif
 
 #undef TEST
 
