@@ -4,19 +4,19 @@
  *
  * Copyright (C) 2011 - Garrett Regier
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU Library General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ * libpeas is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU Library General Public License for more details.
+ * libpeas is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- *  You should have received a copy of the GNU Library General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -35,6 +35,9 @@
 
 #include "extension-c-plugin.h"
 
+/* Used by the local linkage test */
+G_MODULE_EXPORT gpointer global_symbol_clash;
+
 static void introspection_base_iface_init (IntrospectionBaseInterface *iface);
 static void introspection_extension_c_iface_init (IntrospectionCallableInterface *iface);
 static void introspection_has_prerequisite_iface_init (IntrospectionHasPrerequisiteInterface *iface);
@@ -50,6 +53,15 @@ G_DEFINE_DYNAMIC_TYPE_EXTENDED (TestingExtensionCPlugin,
                                 G_IMPLEMENT_INTERFACE_DYNAMIC (INTROSPECTION_TYPE_HAS_PREREQUISITE,
                                                                introspection_has_prerequisite_iface_init))
 
+/* Properties */
+enum {
+  PROP_0,
+  PROP_GLOBAL_SYMBOL_CLASH,
+  N_PROPERTIES
+};
+
+static GParamSpec *properties[N_PROPERTIES] = { NULL };
+
 static void
 testing_extension_c_plugin_init (TestingExtensionCPlugin *plugin)
 {
@@ -76,10 +88,10 @@ testing_extension_c_plugin_call_no_args (IntrospectionCallable *callable)
 {
 }
 
-static const gchar *
+static gchar *
 testing_extension_c_plugin_call_with_return (IntrospectionCallable *callable)
 {
-  return "Hello, World!";
+  return g_strdup ("Hello, World!");
 }
 
 static void
@@ -99,9 +111,38 @@ testing_extension_c_plugin_call_multi_args (IntrospectionCallable *callable,
   *inout = in;
 }
 
+static void
+testing_extension_c_get_property (GObject    *object,
+                                  guint       prop_id,
+                                  GValue     *value,
+                                  GParamSpec *pspec)
+{
+  switch (prop_id)
+    {
+    case PROP_GLOBAL_SYMBOL_CLASH:
+      g_value_set_pointer (value, &global_symbol_clash);
+      break;
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+      break;
+    }
+}
+
 static void
 testing_extension_c_plugin_class_init (TestingExtensionCPluginClass *klass)
 {
+  GObjectClass *object_class = G_OBJECT_CLASS (klass);
+
+  object_class->get_property = testing_extension_c_get_property;
+
+  properties[PROP_GLOBAL_SYMBOL_CLASH] =
+    g_param_spec_pointer ("global-symbol-clash",
+                          "Global symbol clash",
+                          "A global symbol that clashes",
+                          G_PARAM_READABLE |
+                          G_PARAM_STATIC_STRINGS);
+
+  g_object_class_install_properties (object_class, N_PROPERTIES, properties);
 }
 
 static void
