@@ -4,19 +4,19 @@
  *
  * Copyright (C) 2010 - Garrett Regier
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU Library General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ * libpeas is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU Library General Public License for more details.
+ * libpeas is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- *  You should have received a copy of the GNU Library General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -31,24 +31,37 @@
 
 #include "loadable-plugin.h"
 
-struct _TestingLoadablePluginPrivate {
+typedef struct {
   GObject *object;
-};
+} TestingLoadablePluginPrivate;
+
+/* Used by the local linkage test */
+G_MODULE_EXPORT gpointer global_symbol_clash;
 
 static void peas_activatable_iface_init (PeasActivatableInterface *iface);
 
 G_DEFINE_DYNAMIC_TYPE_EXTENDED (TestingLoadablePlugin,
                                 testing_loadable_plugin,
-                                PEAS_TYPE_EXTENSION_BASE,
+                                G_TYPE_OBJECT,
                                 0,
+                                G_ADD_PRIVATE_DYNAMIC (TestingLoadablePlugin)
                                 G_IMPLEMENT_INTERFACE_DYNAMIC (PEAS_TYPE_ACTIVATABLE,
                                                                peas_activatable_iface_init))
 
+#define GET_PRIV(o) \
+  (testing_loadable_plugin_get_instance_private (o))
+
 enum {
   PROP_0,
-  PROP_OBJECT
+  PROP_GLOBAL_SYMBOL_CLASH,
+
+  /* PeasActivatable */
+  PROP_OBJECT,
+  N_PROPERTIES = PROP_OBJECT
 };
 
+static GParamSpec *properties[N_PROPERTIES] = { NULL };
+
 static void
 testing_loadable_plugin_set_property (GObject      *object,
                                       guint         prop_id,
@@ -56,11 +69,12 @@ testing_loadable_plugin_set_property (GObject      *object,
                                       GParamSpec   *pspec)
 {
   TestingLoadablePlugin *plugin = TESTING_LOADABLE_PLUGIN (object);
+  TestingLoadablePluginPrivate *priv = GET_PRIV (plugin);
 
   switch (prop_id)
     {
     case PROP_OBJECT:
-      plugin->priv->object = g_value_get_object (value);
+      priv->object = g_value_get_object (value);
       break;
 
     default:
@@ -76,11 +90,16 @@ testing_loadable_plugin_get_property (GObject    *object,
                                       GParamSpec *pspec)
 {
   TestingLoadablePlugin *plugin = TESTING_LOADABLE_PLUGIN (object);
+  TestingLoadablePluginPrivate *priv = GET_PRIV (plugin);
 
   switch (prop_id)
     {
+    case PROP_GLOBAL_SYMBOL_CLASH:
+      g_value_set_pointer (value, &global_symbol_clash);
+      break;
+
     case PROP_OBJECT:
-      g_value_set_object (value, plugin->priv->object);
+      g_value_set_object (value, priv->object);
       break;
 
     default:
@@ -92,9 +111,6 @@ testing_loadable_plugin_get_property (GObject    *object,
 static void
 testing_loadable_plugin_init (TestingLoadablePlugin *plugin)
 {
-  plugin->priv = G_TYPE_INSTANCE_GET_PRIVATE (plugin,
-                                              TESTING_TYPE_LOADABLE_PLUGIN,
-                                              TestingLoadablePluginPrivate);
 }
 
 static void
@@ -117,7 +133,14 @@ testing_loadable_plugin_class_init (TestingLoadablePluginClass *klass)
 
   g_object_class_override_property (object_class, PROP_OBJECT, "object");
 
-  g_type_class_add_private (klass, sizeof (TestingLoadablePluginPrivate));
+  properties[PROP_GLOBAL_SYMBOL_CLASH] =
+    g_param_spec_pointer ("global-symbol-clash",
+                          "Global symbol clash",
+                          "A global symbol that clashes",
+                          G_PARAM_READABLE |
+                          G_PARAM_STATIC_STRINGS);
+
+  g_object_class_install_properties (object_class, N_PROPERTIES, properties);
 }
 
 static void
